@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011 Radim Rehurek <radimrehurek@seznam.cz>
+# Copyright (C) 2012 Radim Rehurek <radimrehurek@seznam.cz>
 # Licensed under the GNU AGPL v3 - http://www.gnu.org/licenses/agpl.html
 
 
@@ -27,7 +27,6 @@ from __future__ import with_statement
 
 import os
 import logging
-import random
 import threading
 import shutil
 
@@ -49,6 +48,8 @@ SHARD_SIZE = 65536 # spill index shards to disk in SHARD_SIZE-ed chunks of docum
 
 JOURNAL_MODE = 'OFF' # don't keep journals in sqlite dbs
 
+
+
 def merge_sims(oldsims, newsims, clip=TOP_SIMS):
     """Merge two precomputed similarity lists, truncating the result to `clip` most similar items."""
     if oldsims is None:
@@ -222,24 +223,32 @@ class SimIndex(gensim.utils.SaveLoad):
         return result
 
 
+    def vec_by_id(self, docid):
+        """Return indexed vector corresponding to document `docid`."""
+        pos = self.id2pos[docid]
+        return self.qindex.vector_by_id(pos)
+
+
     def sims_by_id(self, docid):
         """Find the most similar documents to the (already indexed) document with `docid`."""
         result = self.id2sims.get(docid, None)
         if result is None:
+            self.qindex.num_best = self.topsims
             sims = self.qindex.similarity_by_id(self.id2pos[docid])
             result = self.sims2scores(sims)
         return result
 
 
-    def sims_by_doc(self, doc, model):
+    def sims_by_vec(self, vec, normalize=None):
         """
-        Find the most similar documents to a fulltext document.
-
-        The document is first processed (tokenized etc) and converted to a vector
-        in the same way the training documents were, during `train()`.
+        Find the most similar documents to a given vector (=already processed document).
         """
-        vec = model.doc2vec(doc) # convert document (text) to vector
-        sims = self.qindex[vec] # query the index
+        if normalize is None:
+            normalize = self.qindex.normalize
+        norm, self.qindex.normalize = self.qindex.normalize, normalize # store old value
+        self.qindex.num_best = self.topsims
+        sims = self.qindex[vec]
+        self.qindex.normalize = norm # restore old value of qindex.normalize
         return self.sims2scores(sims)
 
 
@@ -351,7 +360,7 @@ class SimModel(gensim.utils.SaveLoad):
             self.num_features = len(self.lsi.projection.s)
         elif method == 'logentropy':
             logger.info("training a log-entropy model")
-            self.logent = gensim.models.LogEntropyModel(corpus(), id2word=self.dictionary)
+            self.logent = gensim.models.LogEntropyModel(list(corpus()), id2word=self.dictionary)
             self.num_features = len(self.dictionary)
         else:
             msg = "unknown semantic method %s" % method
@@ -403,7 +412,7 @@ class SimServer(object):
     processes is safe for reading = answering similarity queries. Modifying
     (training/indexing) is realized via locking = serialized internally.
     """
-    def __init__(self, basename):
+    def __init__(self, basename, use_locks=True):
         """
         All data will be stored under directory `basename`. If there is a server
         there already, it will be loaded (resumed).
@@ -414,7 +423,7 @@ class SimServer(object):
         if not os.path.isdir(basename):
             raise ValueError("%r must be a writable directory" % basename)
         self.basename = basename
-        self.lock_update = threading.RLock() # only one thread can modify the server at a time
+        self.lock_update = threading.RLock() if use_locks else gensim.utils.nocm
         try:
             self.fresh_index = SimIndex.load(self.location('index_fresh'))
         except:
@@ -606,7 +615,8 @@ class SimServer(object):
             self.opt_index = SimIndex(self.location('index_opt'), self.model.num_features)
 
         self.opt_index.merge(self.fresh_index)
-        self.fresh_index = self.fresh_index.terminate() # delete old files
+        self.fresh_index.terminate() # delete old files
+        self.fresh_index = None
         self.flush(save_index=True)
 
 
@@ -668,16 +678,23 @@ class SimServer(object):
             docid = doc
             if self.opt_index is not None and docid in self.opt_index:
                 sims_opt = self.opt_index.sims_by_id(docid)
-            if self.fresh_index is not None and docid in self.fresh_index:
+                if self.fresh_index is not None:
+                    vec = self.opt_index.vec_by_id(docid)
+                    sims_fresh = self.fresh_index.sims_by_vec(vec, normalize=False)
+            elif self.fresh_index is not None and docid in self.fresh_index:
                 sims_fresh = self.fresh_index.sims_by_id(docid)
-            if sims_fresh is None and sims_opt is None:
+                if self.opt_index is not None:
+                    vec = self.fresh_index.vec_by_id(docid)
+                    sims_opt = self.opt_index.sims_by_vec(vec, normalize=False)
+            else:
                 raise ValueError("document %r not in index" % docid)
         else:
             # query by an arbitrary text (=tokens) inside doc['tokens']
+            vec = self.model.doc2vec(doc) # convert document (text) to vector
             if self.opt_index is not None:
-                sims_opt = self.opt_index.sims_by_doc(doc, self.model)
+                sims_opt = self.opt_index.sims_by_vec(vec)
             if self.fresh_index is not None:
-                sims_fresh = self.fresh_index.sims_by_doc(doc, self.model)
+                sims_fresh = self.fresh_index.sims_by_vec(vec)
 
         result = []
         for docid, score in merge_sims(sims_opt, sims_fresh):
@@ -736,10 +753,11 @@ class SessionServer(gensim.utils.SaveLoad):
     4. at commit, the clone becomes the original
     5. at rollback, do nothing (clone is discarded, next transaction starts from the original again)
     """
-    def __init__(self, basedir, autosession=True):
+    def __init__(self, basedir, autosession=True, use_locks=True):
         self.basedir = basedir
         self.autosession = autosession
-        self.lock_update = threading.RLock()
+        self.use_locks = use_locks
+        self.lock_update = threading.RLock() if use_locks else gensim.utils.nocm
         self.locs = ['a', 'b'] # directories under which to store stable.session data
         try:
             stable = open(self.location('stable')).read().strip()
@@ -753,7 +771,7 @@ class SessionServer(gensim.utils.SaveLoad):
         except:
             pass
         self.write_istable()
-        self.stable = SimServer(self.loc_stable)
+        self.stable = SimServer(self.loc_stable, use_locks=self.use_locks)
         self.session = None
 
 
@@ -817,8 +835,8 @@ class SessionServer(gensim.utils.SaveLoad):
         logger.info("cloning server from %s to %s" %
                     (self.loc_stable, self.loc_session))
         shutil.copytree(self.loc_stable, self.loc_session)
-        self.session = SimServer(self.loc_session)
-        self.lock_update.acquire() # no other thread can call any modification methods until commit/rollback
+        self.session = SimServer(self.loc_session, use_locks=self.use_locks)
+        #self.lock_update.acquire() # no other thread can call any modification methods until commit/rollback
 
     @gensim.utils.synchronous('lock_update')
     def buffer(self, *args, **kwargs):
@@ -887,7 +905,7 @@ class SessionServer(gensim.utils.SaveLoad):
             self.istable = 1 - self.istable
             self.write_istable()
             tmp.close() # don't wait for gc, release resources manually
-            self.lock_update.release()
+#            self.lock_update.release()
         else:
             logger.warning("commit called but there's no open session in %s" % self)
 
@@ -898,7 +916,7 @@ class SessionServer(gensim.utils.SaveLoad):
             logger.info("rolling back transaction in %s" % self)
             self.session.close()
             self.session = None
-            self.lock_update.release()
+#            self.lock_update.release()
         else:
             logger.warning("rollback called but there's no open session in %s" % self)
 
